/*********************************************************  UIO Driver  *********************************************************
	Red Rapids
	797 N. Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	Description: UIO driver for Red Rapids PCI hardware.

	Rev History
		11/28/2018	-	Added mask bits to interrupt status register read.
		05/17/2018	-	Initial release.
		
	Copyright (c) 2020  Red Rapids, Inc.

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
	files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
	modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
	is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
	LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
	IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*********************************************************  UIO Driver  *********************************************************/

#include <linux/device.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/slab.h>
#include <linux/uio_driver.h>
#include <asm/io.h>

#define RR_PCI_VENDOR_ID 0x17D2
#define RR_PCI_DEVICE_ID 0x00CA
#define GLOBAL_STATUS 0x0100
#define GLOBAL_INT_MASK 0x0108

static irqreturn_t RRadapter_handler(int irq, struct uio_info *dev_info)
{
	void *int_stat = dev_info->mem[0].internal_addr + GLOBAL_STATUS;
	void *int_mask = dev_info->mem[0].internal_addr + GLOBAL_INT_MASK;

	// Return if no active interrupt.
	if (!(ioread32(int_stat) & 0x00001111))
		return IRQ_NONE;

	// Disable all interrupts.
	iowrite32(0x0, int_mask);
	return IRQ_HANDLED;
}

static int RRadapter_probe(struct pci_dev *dev, const struct pci_device_id *id)
{
	struct uio_info *info;

	info = kzalloc(sizeof(struct uio_info), GFP_KERNEL);
	if (!info)
		return -ENOMEM;

	if (pci_enable_device(dev))
		goto out_free;

	if (pci_request_regions(dev, "rrdev"))
		goto out_disable;

	info->mem[0].addr = pci_resource_start(dev, 0);
	if (!info->mem[0].addr)
		goto out_release;
	info->mem[0].internal_addr = pci_ioremap_bar(dev, 0);
	if (!info->mem[0].internal_addr)
		goto out_release;
	info->mem[0].size = pci_resource_len(dev, 0);
	info->mem[0].memtype = UIO_MEM_PHYS;
	
	info->mem[1].addr = pci_resource_start(dev, 2);
	info->mem[1].size = pci_resource_len(dev, 2);
	info->mem[1].memtype = UIO_MEM_PHYS;
	
	info->name = "rrdev";
	info->version = "0.0";
	info->irq = dev->irq;
	info->irq_flags = IRQF_SHARED;
	info->handler = RRadapter_handler;

	if (uio_register_device(&dev->dev, info))
		goto out_unmap;

	return 0;
out_unmap:
	iounmap(info->mem[0].internal_addr);
out_release:
	pci_release_regions(dev);
out_disable:
	pci_disable_device(dev);
out_free:
	kfree (info);
	return -ENODEV;
}

static void RRadapter_remove(struct pci_dev *dev)
{
	struct uio_info *info = pci_get_drvdata(dev);

	uio_unregister_device(info);
	pci_release_regions(dev);
	pci_disable_device(dev);
	pci_set_drvdata(dev, NULL);
	iounmap(info->mem[0].internal_addr);

	kfree (info);
}

static struct pci_device_id RRadapter_pci_ids[] = {
  { PCI_DEVICE(RR_PCI_VENDOR_ID, RR_PCI_DEVICE_ID), },
  { 0, }
};

static struct pci_driver RRadapter_pci_driver = {
	.name = "rrdev",
	.id_table = RRadapter_pci_ids,
	.probe = RRadapter_probe,
	.remove = RRadapter_remove,
};

module_pci_driver(RRadapter_pci_driver);
MODULE_DEVICE_TABLE(pci, RRadapter_pci_ids);
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Red Rapids");
