/*****************************************************  Driver Functions  ******************************************************
	Red Rapids
	797 N Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	Description: Adapter driver function definitions.

	Rev History: 11-28-2018:	Fixed incorrect BAR reference in UIOScan().
				 08-08-2018:	Inital release.

	Copyright (c) 2019  Red Rapids, Inc.

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
	files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
	modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
	is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
	LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
	IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*****************************************************  Driver Functions  ******************************************************/

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>

#include "driver_functions.h"

// Unmap the UIO character device BAR address spaces.
// Return of zero indicates success, non-zero indicates an error condition.
int DEVClose(s_UIOScan UIO, s_DEVMap *p_DEV)
{
	// Delete the BAR0 mapping. 
	if (p_DEV->BAR0 != NULL && p_DEV->BAR0 != MAP_FAILED)
		if (munmap((void *)p_DEV->BAR0, UIO.BAR0Size) < 0)
			return errno;

	// Delete the BAR2 mapping. 
	if (p_DEV->BAR2 != NULL && p_DEV->BAR2 != MAP_FAILED)
		if (munmap((void *)p_DEV->BAR2, UIO.BAR2Size) < 0)
			return errno;
		
	// Close the character file. 
	if (close(p_DEV->fd) < 0)
		return errno;

	return 0;
}

// Initializes all varibles in the device handle structure to default values.
void DEVInit(s_Handle *p_HW, unsigned DevNum)
{
	// Set a unique device number for each hardware handle.
	p_HW->DevNum = DevNum;
	// Set the file descriptor to an illegal value.
	p_HW->DEV.fd = -1;
	// Set all pointers to NULL.
	p_HW->DEV.BAR0 = NULL;
	p_HW->DEV.BAR2 = NULL;
	p_HW->DMA = NULL;
	p_HW->User = NULL;
}
	
// Open the UIO character device file and memory map the BAR address spaces to virtual addresses in user space.
// Return of zero indicates success, non-zero indicates an error condition.
int DEVOpen(s_UIOScan UIO, s_DEVMap *p_DEV)
{
	char uio_path[11];
		
	// Create a string that includes the full directory path name to the requested UIO device character file.
	sprintf(uio_path, "/dev/uio%u", UIO.UIONumber);
					
	// Open the requested UIO device character file.
	if ((p_DEV->fd = open(uio_path, O_RDWR|O_SYNC)) < 0)
		return errno;
	
	// Memory map the BAR0 address space.
	if (UIO.BAR0Size)
		if ((p_DEV->BAR0 = (unsigned long long *)mmap(NULL, UIO.BAR0Size, PROT_READ|PROT_WRITE, MAP_SHARED, p_DEV->fd, 0)) == MAP_FAILED)
			return errno;

	// Memory map the BAR2 address space.
	if (UIO.BAR2Size)
		if ((p_DEV->BAR2 = (unsigned long long *)mmap(NULL, UIO.BAR2Size, PROT_READ|PROT_WRITE, MAP_SHARED, p_DEV->fd, sysconf(_SC_PAGESIZE))) == MAP_FAILED)
			return errno;

	return 0;
}

// Scan all available UIO devices for any Red Rapids device.
// Returns the total number of Red Rapids devices detected.
unsigned UIOScan(s_UIOScan *p_UIO)
{
	FILE *fp;
	unsigned devcnt = 0;
	unsigned uiocnt = 0;
	char uio_path[21];
	char file_name[45];
	char inputline[6];
	
	// Initialize each member of the hardware descriptor structure.
	for (devcnt = 0; devcnt < NUM_DEVICES; devcnt++)
	{
		p_UIO[devcnt].UIONumber = 0;
		p_UIO[devcnt].SubSysID = 0;
		p_UIO[devcnt].BAR0Size = 0;
		p_UIO[devcnt].BAR2Size = 0;
	}
	
	devcnt = 0;
	do
	{
		// Create a string that includes the full path to the next possible UIO directory.
		sprintf(uio_path, "/sys/class/uio/uio%u", uiocnt);
		
		// Create a string that includes the full path to the "name" file.
		strcpy(file_name, uio_path);
		strcat(file_name, "/name");
		
		// Attempt to open the next UIO device "name" file and return if it does not exist.
		if ((fp = fopen(file_name, "r")) == NULL)
			return devcnt;
		// Extract hardware information from the UIO device file system.
		else
			if (fgets(inputline, 6, fp) != NULL)
			{
				// Confirm that this is a Red Rapids device.
				if (!strcmp(inputline, "rrdev"))
				{
					fclose(fp);
					// Record the UIO device number.
					p_UIO[devcnt].UIONumber = devcnt;
					
					// Record the subsystem device ID.
					strcpy(file_name, uio_path);
					strcat(file_name, "/device/subsystem_device");
					if ((fp = fopen(file_name, "r")) == NULL)
						p_UIO[devcnt].SubSysID = 0;
					else
					{
						fscanf(fp, "%x", &p_UIO[devcnt].SubSysID);
						fclose(fp);
					}
					
					// Record the BAR0 size.
					strcpy(file_name, uio_path);
					strcat(file_name, "/maps/map0/size");
					if ((fp = fopen(file_name, "r")) == NULL)
						p_UIO[devcnt].BAR0Size = 0;
					else
					{
						fscanf(fp, "%llx", &p_UIO[devcnt].BAR0Size);
						fclose(fp);
					}
					
					// Record the BAR2 size.
					strcpy(file_name, uio_path);
					strcat(file_name, "/maps/map1/size");
					if ((fp = fopen(file_name, "r")) == NULL)
						p_UIO[devcnt].BAR2Size = 0;
					else
					{
						fscanf(fp, "%llx", &p_UIO[devcnt].BAR2Size);
						fclose(fp);
					}
					
					// Increment the device count.
					devcnt++;
				}
			else
				fclose(fp);
			}

		// Increment the UIO count.
		uiocnt++;
		
	} while (devcnt < NUM_DEVICES);
	
	return devcnt;
}
