/****************************************************  Channel Functions  *****************************************************

	Red Rapids
	797 N Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	The following functions are specific to individual SigStream channels.

	Rev History:  03-11-2019:	Changed ChannelStatus() argument name.
				  02-25-2019:	Inital release.

	Copyright (c) 2019  Red Rapids, Inc.

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
	files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
	modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
	is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
	LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
	IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

****************************************************  Channel Functions  *****************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <errno.h>

#include "driver_functions.h"
#include "sigstream.h"
#include "channel_functions.h"

// Set the channel interrupt mask bits to enable (1) or disable (0) interrupt generation for each event type.
void ChannelMask(s_Handle *p_HW, unsigned ChannelNumber, s_ChannelIntMask ChannelIntMask)
{
	// Write to the channel interrupt mask address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + CHANNEL_INT_MASK] =
				(unsigned long long)ChannelIntMask.DMAMarker |
				(unsigned long long)ChannelIntMask.Error[0] << 4 |
				(unsigned long long)ChannelIntMask.Error[1] << 5;
}

// Issue a reset to restore the channel configuration registers to their power-on state.
void ChannelReset(s_Handle *p_HW, unsigned ChannelNumber)
{
	// Write to the channel reset address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + CHANNEL_RESET] = 0x0ull;
}

// Set all of the channel configuration registers.
int ChannelSettings(s_Handle *p_HW, unsigned ChannelNumber, s_Channel Channel, char *p_File)
{
		// Load the SWITCH SELECT configuration register for the selected channel.
		SwitchSelectSettings(p_HW, ChannelNumber, Channel.SwitchSelect);
		
		// Load the PAYLOAD FORMAT configuration register for the selected channel.
		PayloadFormatSettings(p_HW, ChannelNumber, Channel.PayloadFormat);
		
		// Load the DATAPATH CONTROL configuration register for the selected channel.
		DatapathControlSettings(p_HW, ChannelNumber, Channel.DatapathControl);
		
		// Load the SYNCHRONIZER COMMAND configuration register for the selected channel.
		SynchronizerSettings(p_HW, ChannelNumber, Channel.Synchronizer);
		
		// Load the TOD TRIGGERS configuration register for the selected channel.
		TODTriggerSettings(p_HW, ChannelNumber, Channel.TODTriggers);
		
		// Load the EVENT DURATION configuration register for the selected channel.
		EventDurationSettings(p_HW, ChannelNumber, Channel.EventDuration);
		
		// Load the IF DATA PACKET FORMAT configuration register for the selected channel.
		IFdpktFormatSettings(p_HW, ChannelNumber, Channel.IFdpktFormat);
		
		// Load frame descriptors if the scheduler is active.
		if (Channel.Synchronizer.CycleSelect == 3)
			return SchedulerFrameSettings(p_HW, ChannelNumber, p_File);
		else
			return OK;
}

// Report the channel status and clear the values.
// Returns the total number of status events detected.
int ChannelStatus(s_Handle *p_HW, unsigned ChannelNumber, s_ChannelStatus *p_Channel)
{
	unsigned long long value = 0;
	int status = 0;
	unsigned cnt = 0;
	
	// Save and clear the contents of the channel status register by writing to the error status address.
	// Note: The contents are saved to a shadow register that is accessed during a read operation (below).
	p_HW->DEV.BAR2[(512 * ChannelNumber) + CHANNEL_STATUS] = 0x0ull;
	// Read the saved contents of the channel status.
	value = p_HW->DEV.BAR2[(512 * ChannelNumber) + CHANNEL_STATUS];

	// Strip the DMA marker bit.
	if ((p_Channel->DMAMarker = value & 0x0000000000000001ull)) status++;

	// Strip the individual channel error indicator bits.
	for (cnt = 0; cnt < 2; cnt++)
		if ((p_Channel->Error[cnt] = value >> ((cnt + 4)) & 0x0000000000000001ull)) status++;

	// Return the total fault count.
	return status;
}

// Clear the channel status register.
void ChannelStatusClear(s_Handle *p_HW, unsigned ChannelNumber)
{
	// Write to the channel status address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + CHANNEL_STATUS] = 0x0ull;
}

// Start (OnOff = 1) or stop (OnOff = 0) channel processing.
void FlowControl(s_Handle *p_HW, unsigned ChannelNumber, unsigned OnOff)
{
	// Write to the synchronizer control address.
	if (OnOff)
		p_HW->DEV.BAR2[(512 * ChannelNumber) + SYNCHRONIZER_CTL] = 0x1ull;
	else
		p_HW->DEV.BAR2[(512 * ChannelNumber) + SYNCHRONIZER_CTL] = 0x2ull;
}

// Report the current on/off status of the requested channel.
// Return of zero indicates the channel is off, one indicates the channel is on.
int FlowStatus(s_Handle *p_HW, unsigned ChannelNumber)
{
	// Read the synchronizer control address.
	return (int)(p_HW->DEV.BAR2[(512 * ChannelNumber) + SYNCHRONIZER_CTL] & 0x0000000000000001ull);
}

// Create a frame descriptor file name of frame.txt.
void FrameFilename(char *p_File)
{
	// Note: Assumes files are located in directory ..\files (Windows) or ../files (Linux).
	strcpy(p_File, "..");
	strcat(p_File, SEPARATOR);
	strcat(p_File, "files");
	strcat(p_File, SEPARATOR);
	strcat(p_File, "frame.txt");
}

// Configure the datapath operating controls.
void DatapathControlSettings(s_Handle *p_HW, unsigned ChannelNumber, s_DatapathControl DatapathControl)
{
	// Write to the datapath control address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + DATAPATH_CONTROL] =
			(unsigned long long)(DatapathControl.Transform[1] - 1) |
			(unsigned long long)DatapathControl.Residue[0] << 24 |
			(unsigned long long)DatapathControl.Residue[1] << 28 |
			(unsigned long long)DatapathControl.SyncInitMask[0] << 32 |
			(unsigned long long)DatapathControl.SyncInitMask[1] << 33 |
			(unsigned long long)DatapathControl.SyncInitMask[2] << 34 |
			(unsigned long long)DatapathControl.SyncInitMask[3] << 35 |
			(unsigned long long)DatapathControl.SyncCycleMask[0] << 48 |
			(unsigned long long)DatapathControl.SyncCycleMask[1] << 49 |
			(unsigned long long)DatapathControl.SyncCycleMask[2] << 50 |
			(unsigned long long)DatapathControl.SyncCycleMask[3] << 51;

	// Write to the tuner control address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + TUNER_CONTROL] =
			(-1*(unsigned long long)DatapathControl.Transform[2] & 0x00000000007FFFFFull) |
			((-1*(unsigned long long)DatapathControl.Transform[3] << 32) & 0x007FFFFF00000000ull);

	// Write to the filter control address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + FILTER_CONTROL] =
			(unsigned long long)DatapathControl.Transform[0];
}

// Set the optional event duration timers.
void EventDurationSettings(s_Handle *p_HW, unsigned ChannelNumber, s_EventDuration EventDuration)
{
	// Write to the first event duration address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + EVENT_DURATION1] =
			(unsigned long long)EventDuration.SampleCount;

	// Write to the second event duration address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + EVENT_DURATION2] =
			(unsigned long long)EventDuration.CycleCount |
			((unsigned long long)EventDuration.FrameCount - 3) << 32;
}

// Configure the IF data packet to the requested format.
void IFdpktFormatSettings(s_Handle *p_HW, unsigned ChannelNumber, s_IFdpktFormat IFdpktFormat)
{
	// Write to the first IF data packete format address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + IF_DPKT_FORMAT1] =
			(unsigned long long)IFdpktFormat.PacketSize |
			(unsigned long long)IFdpktFormat.TSFSelect << 16 |
			(unsigned long long)IFdpktFormat.TSISelect << 17 |
			(unsigned long long)IFdpktFormat.TrailerSelect << 18 |
			(unsigned long long)IFdpktFormat.ClassIDSelect << 19 |
			(unsigned long long)IFdpktFormat.StreamIDSelect << 20 |
			(unsigned long long)IFdpktFormat.StreamID << 32;

	// Write to the second IF data packete format address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + IF_DPKT_FORMAT2] =
			(unsigned long long)IFdpktFormat.ClassID;

	// Write to the third IF data packete format address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + IF_DPKT_FORMAT3] =
			(unsigned long long)IFdpktFormat.UserIndicators[0] |
			(unsigned long long)IFdpktFormat.UserIndicators[1] << 1 |
			(unsigned long long)IFdpktFormat.UserIndicators[2] << 2 |
			(unsigned long long)IFdpktFormat.SpectralInversion << 4 |
			(unsigned long long)IFdpktFormat.ContextPktCntEnable << 7 |
			(unsigned long long)IFdpktFormat.IndicatorEnables << 8;
}

// Configure the IF data payload to the requested format.
void PayloadFormatSettings(s_Handle *p_HW, unsigned ChannelNumber, s_PayloadFormat PayloadFormat)
{
	// Write to the payload format address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + PAYLOAD_FORMAT] =
			(unsigned long long)PayloadFormat.PayloadPacking |
			(unsigned long long)PayloadFormat.EventTagOrder << 1 |
			(unsigned long long)PayloadFormat.EventTagMask << 2 |
			(unsigned long long)PayloadFormat.EventTagIndex << 4 |
			(unsigned long long)((PayloadFormat.DataItemSize/2) - 1) << 8 |
			(unsigned long long)((PayloadFormat.ItemPackingFieldSize/2) - 1) << 12 |
			(unsigned long long)PayloadFormat.ChannelTag << 16;
}

// Load frame descriptors for the event scheduler.
int SchedulerFrameSettings(s_Handle *p_HW, unsigned long ChannelNumber, char *p_File)
{
	FILE *fp;
	unsigned long long cnt = 0;
	unsigned long long channel_sync = 0;
	unsigned long long last_slot = 0;
	unsigned long long start_time = 0;
	unsigned long long sample_count = 0;

	// The file consists of alternating start times and sample counts that define a frame.
	if ((fp = fopen(p_File, "r")) == NULL)
		return errno;
	else
	{
		// Get the first two frame descriptors (start time and sample count).
		// Note: Mark an error if there are no descriptors available in the file.
		if (fscanf(fp, "%llu", &start_time) != 1)
		{
			fclose(fp);
			return SCHEDULER_START;
		}
		if (fscanf(fp, "%llu", &sample_count) != 1)
		{
			fclose(fp);
			return SCHEDULER_COUNT;
		}

		// Process all available time slots that define the frame.
		do
		{
			// Set the channel sync bit if the sample count is zero.
			if (!sample_count) channel_sync++; else channel_sync = 0;

			// Write the start time to the holding address.
			p_HW->DEV.BAR2[(512 * ChannelNumber) + SCHEDULER_FRAME1] =
				cnt |
				channel_sync << 12 |
				start_time << 32;
			// Execute a write transaction to scheduler memory zero.
			p_HW->DEV.BAR2[(512 * ChannelNumber) + SCHEDULER_FRAME2] = 0x1ull;

			// Attempt to read another start time to determine whether this is the last time slot.
			if (fscanf(fp, "%llu", &start_time) != 1) last_slot++;

			// Write the sample count to the holding address.
			p_HW->DEV.BAR2[(512 * ChannelNumber) + SCHEDULER_FRAME1] =
				cnt |
				last_slot << 12 |
				sample_count << 32;
			// Execute a write transaction to scheduler memory one.
			p_HW->DEV.BAR2[(512 * ChannelNumber) + SCHEDULER_FRAME2] = 0x2ull;

			// Increment the memory address.
			cnt++;

			// Read another sample count if this was not the last time slot.
			if (!last_slot)
				// Mark an error if there is no sample count defined for the previous time slot.
				if (fscanf(fp, "%llu", &sample_count) != 1)
				{
					fclose(fp);
					return SCHEDULER_COUNT;
				}
		} while (!last_slot && cnt < 512);

		// Close the file.
		fclose(fp);
	}
	return OK;
}

// Select the requested data source for the channel.
void SwitchSelectSettings(s_Handle *p_HW, unsigned ChannelNumber, s_SwitchSelect SwitchSelect)
{
	// Write to the switch select address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + SWITCH_SELECT] =
			(unsigned long long)SwitchSelect.Amplitude |
			(unsigned long long)SwitchSelect.ChannelBypass << 16 |
			(unsigned long long)(SwitchSelect.ChannelSelect - 1) << 18;
}

// Configure the synchronizer operating controls.
void SynchronizerSettings(s_Handle *p_HW, unsigned ChannelNumber, s_Synchronizer Synchronizer)
{
	// Write to the synchronizer command address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + SYNCHRONIZER_CMD] =
			(unsigned long long)Synchronizer.StartEvent |
			(unsigned long long)Synchronizer.StartOffset << 4 |
			(unsigned long long)Synchronizer.StopEvent << 20 |
			(unsigned long long)Synchronizer.StopOffset << 24 |
			(unsigned long long)Synchronizer.CycleSelect << 40 |
			(unsigned long long)Synchronizer.CycleStop << 44;
}

// Set the optional time of day (TOD) start and stop triggers.
void TODTriggerSettings(s_Handle *p_HW, unsigned ChannelNumber, s_TODTriggers TODTriggers)
{
	p_HW->DEV.BAR2[(512 * ChannelNumber) + TOD_TRIGGERS1] =
			(unsigned long long)TODTriggers.FractionalSecondsStart |
			(unsigned long long)TODTriggers.IntegerSecondsStart << 32;

	p_HW->DEV.BAR2[(512 * ChannelNumber) + TOD_TRIGGERS2] =
			(unsigned long long)TODTriggers.FractionalSecondsStop |
			(unsigned long long)TODTriggers.IntegerSecondsStop << 32;
}

// Report the current ready status of the requested channel.
// Return of zero indicates the channel is not ready for a start trigger, one indicates the channel is ready.
int ReadyStatus(s_Handle *p_HW, unsigned ChannelNumber)
{
	// Read the synchronizer control address.
	// Note: Shift the ready status bit so that the function returns either one or zero.
	return (int)((p_HW->DEV.BAR2[(512 * ChannelNumber) + SYNCHRONIZER_CTL] >> 1) & 0x0000000000000001ull);
}

